@@ -32,6 +32,8 @@ import time
 import gettext
 import unicodedata
 import html
+import logging
+import logging.handlers
 import itb_util
 import itb_pango
 
@@ -53,6 +55,8 @@ from gi.repository import GObject
 import itb_emoji
 import version
 
+LOGGER = logging.getLogger('ibus-typing-booster')
+
 GTK_VERSION = (Gtk.get_major_version(),
                Gtk.get_minor_version(),
                Gtk.get_micro_version())
@@ -157,20 +161,6 @@ def parse_args():
         help=('Load only emoji which were added to Unicode '
               + 'not later than this Unicode version. '
               + 'default: %(default)s'))
-    parser.add_argument(
-        '--non_fully_qualified',
-        action='store_true',
-        default=False,
-        help=('Do not use the emoji variation selector '
-              + '(U+FE0F VARIATION SELECTOR-16) '
-              + 'in emoji sequences. Without this option, U+FE0F is used '
-              + 'according to the recommendation of unicode.org. '
-              + 'On some systems, and with some fonts this might make '
-              + 'some emoji sequences render incorrectly though. '
-              + 'If you see some emoji sequences rendered as several glyphs '
-              + 'instead of being rendered as a single glyph, you can try '
-              + 'whether this option improves the rendering for you. '
-              + 'default: %(default)s'))
     parser.add_argument(
         '-d', '--debug',
         action='store_true',
@@ -197,7 +187,6 @@ class EmojiPickerUI(Gtk.Window):
                  unicode_data_all=False,
                  emoji_unicode_min=1.0,
                  emoji_unicode_max=100.0,
-                 non_fully_qualified=False,
                  font=None,
                  fontsize=None,
                  fallback=None):
@@ -270,13 +259,12 @@ class EmojiPickerUI(Gtk.Window):
         self._emoji_unicode_min = emoji_unicode_min
         self._emoji_unicode_max = emoji_unicode_max
         self._unicode_data_all = unicode_data_all
-        self._non_fully_qualified = non_fully_qualified
         self._emoji_matcher = itb_emoji.EmojiMatcher(
             languages=self._languages,
             unicode_data_all=self._unicode_data_all,
             emoji_unicode_min=self._emoji_unicode_min,
             emoji_unicode_max=self._emoji_unicode_max,
-            non_fully_qualified=self._non_fully_qualified)
+            variation_selector='emoji')
         self._gettext_translations = {}
         for language in itb_util.expand_languages(self._languages):
             mo_file = gettext.find(DOMAINNAME, languages=[language])
@@ -504,7 +492,7 @@ class EmojiPickerUI(Gtk.Window):
                 number_of_empty_languages += 1
 
         if _ARGS.debug:
-            sys.stdout.write(
+            LOGGER.debug(
                 'expanded_languages  = %s\n'
                 %itb_util.expand_languages(self._languages)
                 + 'first_language_with_categories = %s\n'
@@ -537,7 +525,7 @@ class EmojiPickerUI(Gtk.Window):
         (dummy_minimum_width_search_bar,
          natural_width_search_bar) = self._search_bar.get_preferred_width()
         if _ARGS.debug:
-            sys.stdout.write(
+            LOGGER.debug(
                 'natural_width_search_entry = %s '
                 %natural_width_search_entry
                 + 'natural_width_search_bar = %s\n'
@@ -564,6 +552,26 @@ class EmojiPickerUI(Gtk.Window):
         self._spinner.stop()
         # self.get_root_window().set_cursor(Gdk.Cursor(Gdk.CursorType.ARROW))
 
+    def _variation_selector_normalize_for_font(self, emoji):
+        '''
+        Normalize the variation selectors in the emoji sequence.
+
+        Returns a new emoji sequences with variation selectors added
+        or removed as appropriate for the current font.
+
+        :param emoji: The emoji
+        :type emoji: String
+        :rtype: String
+        '''
+        if self._font == 'text' or self._font.lower() == 'symbola':
+            return self._emoji_matcher.variation_selector_normalize(
+                emoji, 'text')
+        if self._font == 'unqualified':
+            return self._emoji_matcher.variation_selector_normalize(
+                emoji, '')
+        return self._emoji_matcher.variation_selector_normalize(
+            emoji, 'emoji')
+
     def _browse_treeview_unselect_all(self):
         '''
         Unselect everything in the treeview for browsing the categories
@@ -656,12 +664,14 @@ class EmojiPickerUI(Gtk.Window):
                 self._font + ' ' + str(self._fontsize), emoji,
                 fallback=self._fallback):
             fonts_description += '\n'
+            code_points = ''
             for char in text:
-                fonts_description += (
-                    '<span font="%s" fallback="%s" >'
-                    %(self._font, str(self._fallback).lower())
-                    + char + '</span>'
-                    + ' <span fallback="false">U+%X</span>' %ord(char))
+                code_points += ' U+%X' %ord(char)
+            fonts_description += (
+                '<span font="%s" fallback="%s" >'
+                %(self._font, str(self._fallback).lower())
+                + text + '</span>'
+                + '<span fallback="false">%s</span>' %code_points)
             fonts_description += ': ' + font_family
         descriptions.append(fonts_description)
         if self._emoji_matcher.unicode_version(emoji):
@@ -729,10 +739,9 @@ class EmojiPickerUI(Gtk.Window):
         '''
         (language, label_key, label) = self._currently_selected_label
         if _ARGS.debug:
-            sys.stdout.write(
-                '_fill_flowbox_browse() '
-                + 'language = %s label_key = %s label = %s\n'
-                %(language, label_key, label))
+            LOGGER.debug(
+                'language = %s label_key = %s label = %s\n',
+                language, label_key, label)
         emoji_list = []
         is_recently_used = False
         sorted_recently_used = self._sorted_recently_used()
@@ -757,6 +766,7 @@ class EmojiPickerUI(Gtk.Window):
         for emoji in emoji_list:
             while Gtk.events_pending():
                 Gtk.main_iteration()
+            emoji = self._variation_selector_normalize_for_font(emoji)
             if not is_recently_used:
                 skin_tone_variants = self._emoji_matcher.skin_tone_variants(
                     emoji)
@@ -859,22 +869,20 @@ class EmojiPickerUI(Gtk.Window):
                     mode='r',
                     encoding='UTF-8').read())
             except (PermissionError, SyntaxError, IndentationError):
-                import traceback
-                traceback.print_exc()
+                LOGGER.exception('Error when reading options')
             except Exception as dummy_exception:
-                import traceback
-                traceback.print_exc()
+                LOGGER.exception('Unknown error when reading options')
             else: # no exception occured
                 if _ARGS.debug:
-                    sys.stdout.write(
-                        'File %s has been read and evaluated.\n'
-                        %self._options_file)
+                    LOGGER.debug(
+                        'File %s has been read and evaluated.\n',
+                        self._options_file)
             finally: # executes always
                 if not isinstance(options_dict, dict):
                     if _ARGS.debug:
-                        sys.stdout.write(
-                            'Not a dict: repr(options_dict) = %s\n'
-                            %options_dict)
+                        LOGGER.debug(
+                            'Not a dict: repr(options_dict) = %s\n',
+                            options_dict)
                     options_dict = {}
         if ('font' in options_dict
                 and isinstance(options_dict['font'], str)):
@@ -931,25 +939,22 @@ class EmojiPickerUI(Gtk.Window):
                 traceback.print_exc()
             else: # no exception occured
                 if _ARGS.debug:
-                    sys.stdout.write(
-                        'File %s has been read and evaluated.\n'
-                        %self._recently_used_emoji_file)
+                    LOGGER.debug(
+                        'File %s has been read and evaluated.\n',
+                        self._recently_used_emoji_file)
             finally: # executes always
                 if not isinstance(recently_used_emoji, dict):
                     if _ARGS.debug:
-                        sys.stdout.write(
-                            'Not a dict: '
-                            + 'repr(recently_used_emoji) = %s\n'
-                            %recently_used_emoji)
+                        LOGGER.debug(
+                            'Not a dict: repr(recently_used_emoji) = %s\n',
+                            recently_used_emoji)
                     recently_used_emoji = {}
         self._recently_used_emoji = {}
         for emoji, value in recently_used_emoji.items():
             # add or remove vs16 according to the current setting:
             if emoji:
                 self._recently_used_emoji[
-                    self._emoji_matcher.variation_selector_16_normalize(
-                        emoji,
-                        non_fully_qualified=self._non_fully_qualified)] = value
+                    self._variation_selector_normalize_for_font(emoji)] = value
         if not self._recently_used_emoji:
             self._init_recently_used()
         self._cleanup_recently_used()
@@ -988,7 +993,7 @@ class EmojiPickerUI(Gtk.Window):
         :type _button: Gtk.Button object
         '''
         if _ARGS.debug:
-            sys.stdout.write('on_clear_recently_used_button_clicked()\n')
+            LOGGER.debug('on_clear_recently_used_button_clicked()\n')
         if GTK_VERSION >= (3, 22, 0):
             self._main_menu_popover.popdown()
         self._main_menu_popover.hide()
@@ -1066,14 +1071,16 @@ class EmojiPickerUI(Gtk.Window):
 
     def _print_profiling_information(self):
         '''
-        Print some profiling information to stdout.
+        Print some profiling information to the log.
         '''
         PROFILE.disable()
-        stats = pstats.Stats(PROFILE)
+        stats_stream = io.StringIO()
+        stats = pstats.Stats(PROFILE, stream=stats_stream)
         stats.strip_dirs()
         stats.sort_stats('cumulative')
         stats.print_stats('emoji_picker', 25)
         stats.print_stats('itb_emoji', 25)
+        LOGGER.info('Profiling info:\n%s', stats_stream.getvalue())
 
     def on_delete_event(self, *_args):
         '''
@@ -1103,15 +1110,11 @@ class EmojiPickerUI(Gtk.Window):
         :type event_key: Gdk.EventKey object
         '''
         if _ARGS.debug:
-            sys.stdout.write(
-                'on_main_window_key_press_event() '
-                + 'keyval = %s\n'
-                %event_key.keyval)
+            LOGGER.debug('keyval = %s\n', event_key.keyval)
         if self._fontsize_spin_button.has_focus():
             if _ARGS.debug:
-                sys.stdout.write(
-                    'on_main_window_key_press_event(): '
-                    + 'self._fontsize_spin_button has focus\n')
+                LOGGER.debug(
+                    'self._fontsize_spin_button has focus\n')
             # if the fontsize spin button has focus, we do not want
             # to take it away from there by popping up the search bar.
             return
@@ -1165,7 +1168,7 @@ class EmojiPickerUI(Gtk.Window):
         :type _button: Gtk.Button object
         '''
         if _ARGS.debug:
-            sys.stdout.write('on_about_button_clicked()\n')
+            LOGGER.debug('on_about_button_clicked()\n')
         if GTK_VERSION >= (3, 22, 0):
             self._main_menu_popover.popdown()
         self._main_menu_popover.hide()
@@ -1177,9 +1180,9 @@ class EmojiPickerUI(Gtk.Window):
         and fill the flowbox with the results of the query.
         '''
         if _ARGS.debug:
-            sys.stdout.write(
-                '_fill_flowbox_with_search_results() query_string = %s\n'
-                %self._query_string)
+            LOGGER.debug(
+                '_fill_flowbox_with_search_results() query_string = %s\n',
+                self._query_string)
         candidates = self._emoji_matcher.candidates(
             self._query_string,
             match_limit=1000)
@@ -1203,7 +1206,7 @@ class EmojiPickerUI(Gtk.Window):
             # filled. But then the on_search_entry_search_changed()
             # callback will think that nothing needs to be done
             # because self._candidates_invalid is True already.
-            emoji = candidate[0]
+            emoji = self._variation_selector_normalize_for_font(candidate[0])
             name = candidate[1]
             dummy_score = candidate[2]
             label = Gtk.Label()
@@ -1286,9 +1289,9 @@ class EmojiPickerUI(Gtk.Window):
         :type spin_button: Gtk.SpinButton object
         '''
         if _ARGS.debug:
-            sys.stdout.write(
-                'on_fontsize_spin_button_grab_focus() spin_button = %s\n'
-                %repr(spin_button))
+            LOGGER.debug(
+                'on_fontsize_spin_button_grab_focus() spin_button = %s\n',
+                repr(spin_button))
         spin_button.grab_focus_without_selecting()
         # The default signal handler would again select the contents
         # of the search entry. Therefore, we must prevent the default
@@ -1305,9 +1308,9 @@ class EmojiPickerUI(Gtk.Window):
         :type search_entry: Gtk.SearchEntry object
         '''
         if _ARGS.debug:
-            sys.stdout.write(
-                'on_search_entry_grab_focus() search_entry = %s\n'
-                %repr(search_entry))
+            LOGGER.debug(
+                'on_search_entry_grab_focus() search_entry = %s\n',
+                repr(search_entry))
         search_entry.grab_focus_without_selecting()
         # The default signal handler would again select the contents
         # of the search entry. Therefore, we must prevent the default
@@ -1324,9 +1327,9 @@ class EmojiPickerUI(Gtk.Window):
         '''
         query_string = search_entry.get_text()
         if _ARGS.debug:
-            sys.stdout.write(
-                'on_search_entry_search_changed() query_string = %s\n'
-                %query_string)
+            LOGGER.debug(
+                'on_search_entry_search_changed() query_string = %s\n',
+                query_string)
         if not self._search_bar.get_search_mode():
             # If the text in the search entry changes while
             # the search bar is invisible, ignore it.
@@ -1334,10 +1337,9 @@ class EmojiPickerUI(Gtk.Window):
         self._query_string = query_string
         if self._candidates_invalid:
             if _ARGS.debug:
-                sys.stdout.write(
-                    'on_search_entry_search_changed() '
-                    + 'self._candidates_invalid = %s\n'
-                    %self._candidates_invalid)
+                LOGGER.debug(
+                    'self._candidates_invalid = %s\n',
+                    self._candidates_invalid)
             return
         self._candidates_invalid = True
         self._clear_flowbox()
@@ -1355,7 +1357,7 @@ class EmojiPickerUI(Gtk.Window):
         if not iterator:
             return
         if _ARGS.debug:
-            sys.stdout.write(
+            LOGGER.debug(
                 'on_label_selected(): model[iterator] = %s\n'
                 %model[iterator][:]
                 + 'self._currently_selected_label = %s\n'
@@ -1383,9 +1385,9 @@ class EmojiPickerUI(Gtk.Window):
         :type column: Gtk.TreeViewColumn object
         '''
         if _ARGS.debug:
-            sys.stdout.write(
-                'on_row_activated() %s %s %s\n'
-                %(repr(treeview), repr(treepath), repr(column)))
+            LOGGER.debug(
+                'on_row_activated() %s %s %s\n',
+                repr(treeview), repr(treepath), repr(column))
 
     def _parse_emoji_and_name_from_text( # pylint: disable=no-self-use
             self, text):
@@ -1408,10 +1410,9 @@ class EmojiPickerUI(Gtk.Window):
         if match.group('name'):
             name = html.unescape(match.group('name'))
         if _ARGS.debug:
-            sys.stdout.write(
-                '_parse_emoji_and_name_from_text() '
-                + 'text = %s emoji = %s name = %s\n'
-                %(text, emoji, name))
+            LOGGER.debug(
+                'text = %s emoji = %s name = %s\n',
+                text, emoji, name)
         return (emoji, name)
 
     def _emoji_selected_popover_popdown(self):
@@ -1444,7 +1445,7 @@ class EmojiPickerUI(Gtk.Window):
         # here:
         text = event_box.get_child().get_label()
         if _ARGS.debug:
-            sys.stdout.write("_emoji_event_box_selected() text = %s\n" %text)
+            LOGGER.debug('text = %s\n', text)
         (emoji, name) = self._parse_emoji_and_name_from_text(text)
         if not emoji:
             return Gdk.EVENT_PROPAGATE
@@ -1484,7 +1485,7 @@ class EmojiPickerUI(Gtk.Window):
         :type flowbox_child: Gtk.FlowBoxChild object
         '''
         if _ARGS.debug:
-            sys.stdout.write("on_emoji_selected()\n")
+            LOGGER.debug("on_emoji_selected()\n")
         event_box = flowbox_child.get_child()
         self._emoji_event_box_selected(event_box)
         return Gdk.EVENT_PROPAGATE
@@ -1497,7 +1498,7 @@ class EmojiPickerUI(Gtk.Window):
         :type _button: Gtk.Button object
         '''
         if _ARGS.debug:
-            sys.stdout.write('on_main_menu_button_clicked()\n')
+            LOGGER.debug('on_main_menu_button_clicked()\n')
         if GTK_VERSION >= (3, 22, 0):
             self._main_menu_popover.popup()
         self._main_menu_popover.show_all()
@@ -1510,7 +1511,7 @@ class EmojiPickerUI(Gtk.Window):
         :type  _button: Gtk.Button object
         '''
         if _ARGS.debug:
-            sys.stdout.write('on_toggle_search_button_clicked()\n')
+            LOGGER.debug('on_toggle_search_button_clicked()\n')
         self._search_bar.set_search_mode(
             not self._search_bar.get_search_mode())
 
@@ -1524,6 +1525,7 @@ class EmojiPickerUI(Gtk.Window):
             text = label.get_label()
             (emoji, name) = self._parse_emoji_and_name_from_text(text)
             if emoji:
+                emoji = self._variation_selector_normalize_for_font(emoji)
                 new_text = (
                     '<span font="%s %s" fallback="%s">'
                     %(self._font, self._fontsize, str(self._fallback).lower())
@@ -1531,7 +1533,7 @@ class EmojiPickerUI(Gtk.Window):
                     + '</span>')
                 if name:
                     new_text += (
-                        '<span fallback="false" font="%s">'
+                        '<span fallback="true" font="%s">'
                         %(self._fontsize / 2)
                         + html.escape(name)
                         + '</span>')
@@ -1573,7 +1575,7 @@ class EmojiPickerUI(Gtk.Window):
         # here:
         text = flowbox_child.get_child().get_label()
         if _ARGS.debug:
-            sys.stdout.write('on_skin_tone_selected() text = %s\n' %text)
+            LOGGER.debug('on_skin_tone_selected() text = %s\n', text)
         (emoji, dummy_name) = self._parse_emoji_and_name_from_text(text)
         if not emoji:
             return
@@ -1620,9 +1622,9 @@ class EmojiPickerUI(Gtk.Window):
         :type event_sequence: Gdk.EventSequence structure
         '''
         if _ARGS.debug:
-            sys.stdout.write(
-                'on_flowbox_event_box_long_press_begin() %s %s\n'
-                %(repr(gesture), repr(event_sequence)))
+            LOGGER.debug(
+                'on_flowbox_event_box_long_press_begin() %s %s\n',
+                repr(gesture), repr(event_sequence))
 
     def on_flowbox_event_box_long_press_cancel(self, gesture, event_sequence):
         '''
@@ -1634,9 +1636,9 @@ class EmojiPickerUI(Gtk.Window):
         :type event_sequence: Gdk.EventSequence structure
         '''
         if _ARGS.debug:
-            sys.stdout.write(
-                'on_flowbox_event_box_long_press_cancel() %s %s\n'
-                %(repr(gesture), repr(event_sequence)))
+            LOGGER.debug(
+                'on_flowbox_event_box_long_press_cancel() %s %s\n',
+                repr(gesture), repr(event_sequence))
 
     def on_flowbox_event_box_long_press_cancelled(self, gesture):
         '''
@@ -1647,9 +1649,9 @@ class EmojiPickerUI(Gtk.Window):
         :type gesture: Gtk.GestureLongPress object
         '''
         if _ARGS.debug:
-            sys.stdout.write(
-                'on_flowbox_event_box_long_press_cancelled() %s\n'
-                %repr(gesture))
+            LOGGER.debug(
+                'on_flowbox_event_box_long_press_cancelled() %s\n',
+                repr(gesture))
 
     def on_flowbox_event_box_long_press_pressed(
             self, gesture, x, y, event_box):
@@ -1664,9 +1666,9 @@ class EmojiPickerUI(Gtk.Window):
         :type y: Integer
         '''
         if _ARGS.debug:
-            sys.stdout.write(
-                'on_flowbox_event_box_long_press_pressed() %s %s %s %s\n'
-                %(repr(gesture), x, y, repr(event_box)))
+            LOGGER.debug(
+                'on_flowbox_event_box_long_press_pressed() %s %s %s %s\n',
+                repr(gesture), x, y, repr(event_box))
         self._show_skin_tone_popover(event_box)
 
     def on_flowbox_event_box_button_release(
@@ -1680,7 +1682,7 @@ class EmojiPickerUI(Gtk.Window):
         :type event_button: Gdk.EventButton object
         '''
         if _ARGS.debug:
-            sys.stdout.write(
+            LOGGER.debug(
                 'on_flowbox_event_box_button_release() '
                 + 'event_button.type=%s ' %event_button.type
                 + 'event_button.window=%s ' %event_button.window
@@ -1719,8 +1721,11 @@ class EmojiPickerUI(Gtk.Window):
         (emoji, name) = self._parse_emoji_and_name_from_text(text)
         if not emoji:
             return
-        skin_tone_variants = self._emoji_matcher.skin_tone_variants(emoji)
-        if len(skin_tone_variants) == 1:
+        skin_tone_variants = []
+        for skin_tone_variant in self._emoji_matcher.skin_tone_variants(emoji):
+            skin_tone_variants.append(
+                self._variation_selector_normalize_for_font(skin_tone_variant))
+        if len(skin_tone_variants) <= 1:
             return
         self._skin_tone_popover = Gtk.Popover()
         self._skin_tone_popover.set_modal(True)
@@ -1947,7 +1952,7 @@ class EmojiPickerUI(Gtk.Window):
 
         '''
         if _ARGS.debug:
-            sys.stdout.write(
+            LOGGER.debug(
                 'on_flowbox_event_box_button_press() '
                 + 'event_button.type=%s ' %event_button.type
                 + 'event_button.window=%s ' %event_button.window
@@ -1980,9 +1985,8 @@ class EmojiPickerUI(Gtk.Window):
         '''
         value = adjustment.get_value()
         if _ARGS.debug:
-            sys.stdout.write(
-                'on_fontsize_adjustment_value_changed() value = %s\n'
-                %value)
+            LOGGER.debug(
+                'on_fontsize_adjustment_value_changed() value = %s\n', value)
         self._fontsize = value
         self._save_options()
         self._busy_start()
@@ -1998,9 +2002,9 @@ class EmojiPickerUI(Gtk.Window):
         '''
         self._fallback = check_button.get_active()
         if _ARGS.debug:
-            sys.stdout.write(
-                'on_fallback_check_button_toggled() self._fallback = %s\n'
-                %self._fallback)
+            LOGGER.debug(
+                'on_fallback_check_button_toggled() self._fallback = %s\n',
+                self._fallback)
         self._save_options()
         self._busy_start()
         GLib.idle_add(self._change_flowbox_font)
@@ -2017,13 +2021,19 @@ class EmojiPickerUI(Gtk.Window):
             'Apple Color Emoji 🎨', # color
             'Emoji Two 🎨', # color
             'Emoji One 🎨', # color
+            'JoyPixels 🎨', # color
             'Symbola 🙾', # black and white
             'Noto Emoji 🙾', # black and white
             'Android Emoji 🙾', # black and white
             'Segoe UI Emoji 🙾', # seems to be black and white
             'Twitter Color Emoji 🙾', # seems to be black and white
         ]
-        available_good_emoji_fonts = ['emoji (' + _('System default') + ')']
+        available_good_emoji_fonts = [
+            'emoji (' + _('System default') + ')',
+            'text (' + _('System default') + ')',
+            'unqualified (' + _('System default') + ')',
+        ]
+        available_good_emoji_fonts.append('')
         pango_context = self.get_pango_context()
         families = pango_context.list_families()
         names = sorted([family.get_name() for family in families])
@@ -2048,9 +2058,8 @@ class EmojiPickerUI(Gtk.Window):
         :type filter_text: String
         '''
         if _ARGS.debug:
-            sys.stdout.write(
-                '_fill_listbox_font() filter_text = %s\n'
-                %filter_text)
+            LOGGER.debug(
+                '_fill_listbox_font() filter_text = %s\n', filter_text)
         for child in self._font_popover_scroll.get_children():
             self._font_popover_scroll.remove(child)
         self._font_popover_listbox = Gtk.ListBox()
@@ -2089,9 +2098,9 @@ class EmojiPickerUI(Gtk.Window):
         '''
         filter_text = search_entry.get_text()
         if _ARGS.debug:
-            sys.stdout.write(
-                'on_font_search_entry_search_changed() filter_text = %s\n'
-                %filter_text)
+            LOGGER.debug(
+                'on_font_search_entry_search_changed() filter_text = %s\n',
+                filter_text)
         self._fill_listbox_font(filter_text)
 
     def on_font_selected(self, _listbox, listbox_row):
@@ -2105,11 +2114,10 @@ class EmojiPickerUI(Gtk.Window):
         '''
         font = listbox_row.get_child().get_text().split(' ')[0]
         if _ARGS.debug:
-            sys.stdout.write(
-                'on_font_selected() font = %s\n' %repr(font))
+            LOGGER.debug('on_font_selected() font = %s\n', repr(font))
         if font == '':
             font = 'emoji'
-        if font != self._font and font == 'emoji':
+        if font != self._font and font in ('emoji', 'text', 'unqualified'):
             self._fallback = True
             self._fallback_check_button.set_active(True)
         else:
@@ -2132,7 +2140,7 @@ class EmojiPickerUI(Gtk.Window):
         :type _button: Gtk.Button object
         '''
         if _ARGS.debug:
-            sys.stdout.write(
+            LOGGER.debug(
                 'on_font_button_clicked()\n')
         self._font_popover = Gtk.Popover()
         self._font_popover.set_relative_to(self._font_button)
@@ -2194,10 +2202,21 @@ def get_languages():
     return languages
 
 if __name__ == '__main__':
-    if _ARGS.debug:
-        sys.stdout.write('--- %s ---\n' %time.strftime('%Y-%m-%d: %H:%M:%S'))
+    if not _ARGS.debug:
+        LOG_HANDLER = logging.NullHandler()
+    else:
+        LOG_HANDLER = logging.StreamHandler(stream=sys.stdout)
+        LOG_FORMATTER = logging.Formatter(
+            '%(asctime)s %(filename)s '
+            'line %(lineno)d %(funcName)s %(levelname)s: '
+            '%(message)s')
+        LOG_HANDLER.setFormatter(LOG_FORMATTER)
+        LOGGER.setLevel(logging.DEBUG)
+        LOGGER.addHandler(LOG_HANDLER)
+        LOGGER.info('********** STARTING **********')
         import cProfile
         import pstats
+        import io
         PROFILE = cProfile.Profile()
         PROFILE.enable()
 
@@ -2208,7 +2227,7 @@ if __name__ == '__main__':
     try:
         locale.setlocale(locale.LC_ALL, '')
     except locale.Error:
-        sys.stderr.write("IBUS-WARNING **: Using the fallback 'C' locale")
+        LOGGER.error("IBUS-WARNING **: Using the fallback 'C' locale")
         locale.setlocale(locale.LC_ALL, 'C')
 
     LOCALEDIR = os.getenv("IBUS_LOCALEDIR")
@@ -2227,6 +2246,5 @@ if __name__ == '__main__':
         modal=_ARGS.modal,
         unicode_data_all=_ARGS.all,
         emoji_unicode_min=_ARGS.emoji_unicode_min,
-        emoji_unicode_max=_ARGS.emoji_unicode_max,
-        non_fully_qualified=_ARGS.non_fully_qualified)
+        emoji_unicode_max=_ARGS.emoji_unicode_max)
     Gtk.main()
