@@ -27,8 +27,11 @@ hunspell dictonaries.
 import os
 import sys
 import unicodedata
+import logging
 import itb_util
 
+LOGGER = logging.getLogger('ibus-typing-booster')
+
 DEBUG_LEVEL = int(0)
 
 IMPORT_ENCHANT_SUCCESSFUL = False
@@ -56,8 +59,7 @@ class Dictionary:
     '''
     def __init__(self, name='en_US'):
         if DEBUG_LEVEL > 1:
-            sys.stderr.write(
-                "Dictionary.__init__(name=%s)\n" %name)
+            LOGGER.debug('Dictionary.__init__(name=%s)\n', name)
         self.name = name
         self.dic_path = ''
         self.encoding = 'UTF-8'
@@ -74,7 +76,7 @@ class Dictionary:
 
         '''
         if DEBUG_LEVEL > 0:
-            sys.stderr.write("load_dictionary() ...\n")
+            LOGGER.debug('load_dictionary() ...\n')
         (self.dic_path,
          self.encoding,
          self.words) = itb_util.get_hunspell_dictionary_wordlist(self.name)
@@ -100,15 +102,34 @@ class Dictionary:
                 if len(word) > self.max_word_len:
                     self.max_word_len = len(word)
             if DEBUG_LEVEL > 1:
-                sys.stderr.write(
-                    'load_dictionary() max_word_len = %s\n'
-                    % self.max_word_len)
+                LOGGER.debug(
+                    'max_word_len = %s\n', self.max_word_len)
             if IMPORT_ENCHANT_SUCCESSFUL:
-                self.enchant_dict = enchant.Dict(self.name)
+                try:
+                    self.enchant_dict = enchant.Dict(self.name)
+                except enchant.errors.DictNotFoundError:
+                    LOGGER.exception(
+                        'Error initializing enchant for %s', self.name)
+                    self.enchant_dict = None
+                except Exception:
+                    LOGGER.exception(
+                        'Unknown error initializing enchant for %s',
+                        self.name)
+                    self.enchant_dict = None
             elif IMPORT_HUNSPELL_SUCCESSFUL and self.dic_path:
                 aff_path = self.dic_path.replace('.dic', '.aff')
-                self.pyhunspell_object = hunspell.HunSpell(
-                    self.dic_path, aff_path)
+                try:
+                    self.pyhunspell_object = hunspell.HunSpell(
+                        self.dic_path, aff_path)
+                except hunspell.HunSpellError:
+                    LOGGER.debug(
+                        'Error initializing hunspell for %s', self.name)
+                    self.pyhunspell_object = None
+                except Exception:
+                    LOGGER.debug(
+                        'Unknown error initializing hunspell for %s',
+                        self.name)
+                    self.pyhunspell_object = None
 
 class Hunspell:
     '''A class to suggest completions or corrections
@@ -122,12 +143,11 @@ class Hunspell:
             DEBUG_LEVEL = int(0)
         if DEBUG_LEVEL > 1:
             if dictionary_names:
-                sys.stderr.write(
-                    'Hunspell.__init__(dictionary_names=%s)\n'
-                    %dictionary_names)
+                LOGGER.debug(
+                    'Hunspell.__init__(dictionary_names=%s)\n',
+                    dictionary_names)
             else:
-                sys.stderr.write(
-                    'Hunspell.__init__(dictionary_names=())\n')
+                LOGGER.debug('Hunspell.__init__(dictionary_names=())\n')
         self._suggest_cache = {}
         self._dictionary_names = dictionary_names
         self._dictionaries = []
@@ -138,11 +158,11 @@ class Hunspell:
         '''
         if DEBUG_LEVEL > 1:
             if self._dictionary_names:
-                sys.stderr.write(
-                    'Hunspell.init_dictionaries() dictionary_names=%s\n'
-                    %self._dictionary_names)
+                LOGGER.debug(
+                    'Hunspell.init_dictionaries() dictionary_names=%s\n',
+                    self._dictionary_names)
             else:
-                sys.stderr.write(
+                LOGGER.debug(
                     'Hunspell.init_dictionaries() dictionary_names=()\n')
         self._suggest_cache = {}
         self._dictionaries = []
@@ -160,7 +180,7 @@ class Hunspell:
         '''Sets the list of dictionary names.
 
         If the new list of dictionary names differs from the existing
-        one, re-initilize the dictionaries.
+        one, re-initialize the dictionaries.
         '''
         if dictionary_names != self._dictionary_names:
             if set(dictionary_names) != set(self._dictionary_names):
@@ -179,9 +199,9 @@ class Hunspell:
                             dictionaries_new.append(dictionary)
                 self._dictionaries = dictionaries_new
         if DEBUG_LEVEL > 1:
-            sys.stderr.write('set_dictionary_names(%s):\n' % dictionary_names)
+            LOGGER.debug('set_dictionary_names(%s):\n', dictionary_names)
             for dictionary in self._dictionaries:
-                sys.stderr.write('%s\n' % dictionary.name)
+                LOGGER.debug('%s\n', dictionary.name)
 
     def suggest(self, input_phrase):
         # pylint: disable=line-too-long
@@ -236,14 +256,22 @@ class Hunspell:
 
         >>> h.suggest('tenéis')[0]
         ('tenéis', 0)
+
+        >>> h = Hunspell(['en_US'])
+        >>> h.suggest('camel')
+        [('camel', 0), ('camellia', 0), ('camelhair', 0), ('came', -1), ('Camel', -1), ('cameo', -1), ('came l', -1), ('camels', -1)]
+
+        >>> h = Hunspell(['fr_FR'])
+        >>> h.suggest('differemmen')
+        [('différemment', 0)]
         '''
         # pylint: enable=line-too-long
         if input_phrase in self._suggest_cache:
             return self._suggest_cache[input_phrase]
         if DEBUG_LEVEL > 1:
-            sys.stderr.write(
-                "Hunspell.suggest() input_phrase=%(ip)s\n"
-                %{'ip': input_phrase.encode('UTF-8')})
+            LOGGER.debug(
+                "Hunspell.suggest() input_phrase=%(ip)s\n",
+                {'ip': input_phrase.encode('UTF-8')})
         # http://pwet.fr/man/linux/fichiers_speciaux/hunspell says:
         #
         # > A dictionary file (*.dic) contains a list of words, one per
